from base64 import b64decode, b64encode
from datetime import datetime, timedelta
from json import dumps
from ..utils.exceptions import KmsError
from ..utils.microservices import Microservice
from ..utils.metadata import Metadata
from ..utils.request import GET, POST, Request


class KmsGateway(object):

    @classmethod
    def decrypt(cls, encryptedData, keyId):
        accessToken = Metadata.getServiceAccessToken()
        response = Request.fetch(
            url="https://cloudkms.googleapis.com/v1/{keyId}:decrypt".format(
                keyId=keyId,
            ),
            method=POST,
            body=dumps({"ciphertext": encryptedData}),
            headers={
                "Content-Type": "application/json",
                "Authorization": "Bearer {}".format(accessToken)
            }
        )

        if response.status > 299:
            raise KmsError(response.contentJson())

        encodedData = response.contentJson()["plaintext"]

        data = b64decode(encodedData)

        return data

    @classmethod
    def encrypt(cls, payload, keyId):
        accessToken = Metadata.getServiceAccessToken()
        response = Request.fetch(
            url="https://cloudkms.googleapis.com/v1/{keyId}:encrypt".format(
                keyId=keyId,
            ),
            method=POST,
            body=dumps({"plaintext": b64encode(payload)}),
            headers={
                "Content-Type": "application/json",
                "Authorization": "Bearer {}".format(accessToken)
            }
        )

        if response.status > 299:
            raise KmsError(response.contentJson())

        encodedData = response.contentJson()["ciphertext"]

        return encodedData

    @classmethod
    def buildKeyId(cls, keyring, keyName, location="southamerica-east1"):
        return "projects/{project}/locations/{location}/keyRings/{keyring}/cryptoKeys/{key}".format(
            project=Microservice.getProjectId(Microservice.auth),
            location=location,
            keyring=keyring,
            key=keyName
        )

    @classmethod
    def getKeyById(cls, keyId):
        accessToken = Metadata.getServiceAccessToken()
        response = Request.fetch(
            url="https://cloudkms.googleapis.com/v1/{keyId}".format(
                keyId=keyId
            ),
            method=GET,
            headers={
                "Authorization": "Bearer {}".format(accessToken)
            }
        )

        if response.status == 200:
            return response.contentJson()["name"]

        if response.status != 404:
            raise KmsError(response.contentJson())

        return None

    @classmethod
    def createKeyById(cls, keyId):
        accessToken = Metadata.getServiceAccessToken()
        parentPath, keyName = keyId.rsplit("/", 1)

        response = Request.fetch(
            url="https://cloudkms.googleapis.com/v1/{parentPath}?cryptoKeyId={keyName}".format(
                parentPath=parentPath,
                keyName=keyName
            ),
            method=POST,
            body=dumps({
                "purpose": "ENCRYPT_DECRYPT",
                "rotationPeriod": "2592000s",
                "nextRotationTime": (datetime.utcnow() + timedelta(days=30)).isoformat() + "Z"
            }),
            headers={
                "Content-Type": "application/json",
                "Authorization": "Bearer {}".format(accessToken)
            }
        )

        if response.status != 200:
            raise KmsError(response.contentJson())

        return response.contentJson()["name"]

    @classmethod
    def ensureKeyExists(cls, keyId):
        foundKey = cls.getKeyById(keyId=keyId)

        if foundKey:
            return foundKey

        createdKey = cls.createKeyById(keyId=keyId)
        return createdKey
