from ..models.shardedCounter import Shard, ShardedCounter
from google.appengine.api.datastore_errors import TransactionFailedError
from google.appengine.ext import ndb
from random import randint

shardKeyTemplate = 'shard-{counter}-{shard:d}'


class ShardedCounterGateway:

    @classmethod
    def offsetCounter(cls, counterName, delta):
        cls.offsetCounterAsync(counterName, delta=delta).get_result()

    @classmethod
    @ndb.tasklet
    def offsetCounterAsync(cls, counterName, delta):
        if delta == 0:
            raise ndb.Return(None)

        numberOfShards = yield cls._getNumberOfShards(counterName)
        shardNumber = randint(1, numberOfShards)
        try:
            yield cls._offsetShard(counterName=counterName, shardNumber=shardNumber, delta=delta)
        except TransactionFailedError:
            shardNumber = yield cls._addNewShard(counterName)
            yield cls._offsetShard(counterName=counterName, shardNumber=shardNumber, delta=delta)

    @classmethod
    def getAccumulated(cls, counterName):
        return cls.getAccumulatedAsync(counterName).get_result()

    @classmethod
    @ndb.tasklet
    def getAccumulatedAsync(cls, counterName):
        delta = yield cls.getDeltaAsync(counterName)
        accumulated = cls._getAccumulated(counterName)
        raise ndb.Return(delta + accumulated)

    @classmethod
    def getDelta(cls, counterName):
        return cls.getDeltaAsync(counterName).get_result()

    @classmethod
    @ndb.tasklet
    def getDeltaAsync(cls, counterName):
        shardKeys = yield cls._getAllShardKeys(counterName)
        shards = yield ndb.get_multi_async(shardKeys)
        total = sum(shard.value for shard in shards if shard)
        raise ndb.Return(total)

    @classmethod
    def accumulateDelta(cls, counterName, delta):
        return cls.accumulateDeltaAsync(counterName=counterName, delta=delta).get_result()

    @classmethod
    @ndb.transactional_tasklet(xg=True, propagation=ndb.TransactionOptions.ALLOWED)
    def accumulateDeltaAsync(cls, counterName, delta):
        if delta == 0:
            raise ndb.Return(None)

        shardCounter = yield ShardedCounter.getByIdAsync(counterName, useCache=False)

        yield cls.offsetCounterAsync(counterName=counterName, delta=-delta)
        shardCounter.accumulated += delta
        shardCounter.save()

    @classmethod
    @ndb.transactional_async(xg=True, propagation=ndb.TransactionOptions.ALLOWED)
    def _addNewShard(cls, counterName):
        shardCounter = ShardedCounter.getById(counterName, useCache=False)
        if not shardCounter:
            shardCounter = ShardedCounter(id=counterName)
        shardCounter.numShards += 1
        shardCounter.save()
        return shardCounter.numShards

    @classmethod
    @ndb.transactional_async(xg=True, propagation=ndb.TransactionOptions.ALLOWED)
    def _offsetShard(cls, counterName, shardNumber, delta=1):
        shard = cls._getShard(counterName, shardNumber).get_result()
        shard.value += delta
        shard.save()

    @classmethod
    @ndb.tasklet
    def _getShard(cls, counterName, index):
        shardName = shardKeyTemplate.format(
            counter=counterName,
            shard=index,
        )
        shard = yield Shard.getByIdAsync(shardName, useCache=False)
        if not shard:
            shard = Shard(id=shardName)
        raise ndb.Return(shard)

    @classmethod
    @ndb.tasklet
    def _getAllShardKeys(cls, counterName):
        numberOfShards = yield cls._getNumberOfShards(counterName)
        raise ndb.Return([
            ndb.Key(Shard, shardKeyTemplate.format(
                counter=counterName,
                shard=index + 1,
            ))
            for index in xrange(numberOfShards)
        ])

    @classmethod
    @ndb.tasklet
    def _getAccumulated(cls, counterName):
        shardCounter = yield ShardedCounter.getByIdAsync(counterName, useCache=False)
        if shardCounter:
            raise ndb.Return(shardCounter.accumulated)
        raise ndb.Return(0)

    @classmethod
    @ndb.tasklet
    def _getNumberOfShards(cls, counterName):
        shardCounter = yield ShardedCounter.getByIdAsync(counterName, useCache=False)
        if not shardCounter:
            shardCounter = ShardedCounter(id=counterName)
            yield shardCounter.saveAsync()
        raise ndb.Return(shardCounter.numShards)
