from ..utils.enum import Enum


class Environment(Enum):

    sandbox = "sandbox"
    production = "production"
    development = "development"

    @classmethod
    def current(cls):
        from ..utils.metadata import Metadata
        projectId = Metadata.getProjectId()
        if "-dev" in projectId: return cls.development
        if "-sbx" in projectId: return cls.sandbox
        return cls.production


class Microservice(Enum):

    billing = "billing"
    chargePayment = "charge-payment"
    taxPayment = "tax-payment"
    ditto = "ditto"
    email = "email"
    webhook = "webhook"
    address = "address"
    workspace = "workspace"
    transfer = "transfer"
    gateway = "gateway"
    charge = "charge"
    team = "team"
    bank = "bank"
    auth = "auth"
    edi = "edi"
    pdf = "pdf"
    nfe = "nfe"
    holmes = "holmes"
    approval = "approval"

    @classmethod
    def getBaseUrl(cls, ms, environment=None, followVersion=True):
        return "https://{host}".format(host=cls.getBaseHost(ms, environment=environment, followVersion=followVersion))

    @classmethod
    def getBaseHost(cls, ms, environment=None, followVersion=True):
        from ..utils.metadata import Metadata
        projectId = cls.getProjectId(ms, environment=environment or Environment.current())
        version = Metadata.getRunningVersion()
        projectId = "{version}-dot-{projectId}".format(
            version=version,
            projectId=projectId,
        ) if version and followVersion else projectId
        return "{projectId}.rj.r.appspot.com".format(projectId=projectId)

    @classmethod
    def getLogoUrl(cls):
        return "https://starkbank.com/images/logo.png"

    @classmethod
    def getProjectId(cls, ms, environment=None):
        return {
            Environment.production:  _productionProjectId.get(ms),
            Environment.sandbox:     _sandboxProjectId.get(ms),
            Environment.development: _developmentProjectId.get(ms)
        }[environment or Environment.current()]


class Webapp:

    @classmethod
    def getBaseUrl(cls, environment=None):
        return {
            Environment.production:  "https://web.starkbank.com",
            Environment.sandbox:     "https://sandbox.web.starkbank.com",
            Environment.development: "https://development.web.starkbank.com"
        }[environment or Environment.current()]


_developmentProjectId = {
    Microservice.transfer:      "api-ms-transfer-dev",
    Microservice.charge:        "api-ms-charge-dev",
    Microservice.team:          "api-ms-refund-dev",
    Microservice.workspace:     "api-ms-workspace-dev",
    Microservice.bank:          "api-ms-bank-dev",
    Microservice.gateway:       "api-ms-gateway-dev",
    Microservice.edi:           "api-ms-edi-dev",
    Microservice.address:       "api-ms-address-dev",
    Microservice.auth:          "api-ms-auth-dev",
    Microservice.webhook:       "api-ms-webhook-dev",
    Microservice.pdf:           "api-ms-pdf-dev",
    Microservice.chargePayment: "api-ms-charge-payment-dev",
    Microservice.billing:       "api-ms-billing-dev",
    Microservice.ditto:         "api-ms-ditto-dev",
    Microservice.email:         "api-ms-email-dev",
    Microservice.nfe:           "api-ms-nfe-dev",
    Microservice.taxPayment:    "api-ms-tax-payment-dev",
    Microservice.holmes:        "api-ms-holmes-dev",
    Microservice.approval:      "api-ms-approval-dev",
}

_sandboxProjectId = {
    Microservice.transfer:      "api-ms-transfer-sbx",
    Microservice.charge:        "api-ms-charge-sbx",
    Microservice.team:          "api-ms-refund-sbx",
    Microservice.workspace:     "api-ms-workspace-sbx",
    Microservice.bank:          "api-ms-bank-sbx",
    Microservice.gateway:       "api-ms-gateway-sbx",
    Microservice.edi:           "api-ms-edi-sbx",
    Microservice.address:       "api-ms-address-sbx",
    Microservice.auth:          "api-ms-auth-sbx",
    Microservice.webhook:       "api-ms-webhook-sbx",
    Microservice.pdf:           "api-ms-pdf-sbx",
    Microservice.chargePayment: "api-ms-charge-payment-sbx",
    Microservice.billing:       "api-ms-billing-sbx",
    Microservice.ditto:         "api-ms-ditto-sbx",
    Microservice.email:         "api-ms-email-sbx",
    Microservice.nfe:           "api-ms-nfe-sbx",
    Microservice.taxPayment:    "api-ms-tax-payment-sbx",
    Microservice.holmes:        "api-ms-holmes-sbx",
    Microservice.approval:      "api-ms-approval-sbx",
}

_productionProjectId = {
    Microservice.transfer:      "api-ms-transfer",
    Microservice.charge:        "api-ms-charge",
    Microservice.team:          "api-ms-refund",
    Microservice.workspace:     "api-ms-workspace",
    Microservice.bank:          "api-ms-bank",
    Microservice.gateway:       "api-ms-gateway",
    Microservice.edi:           "api-ms-edi",
    Microservice.address:       "api-ms-address",
    Microservice.auth:          "api-ms-auth",
    Microservice.webhook:       "api-ms-webhook",
    Microservice.pdf:           "api-ms-pdf",
    Microservice.chargePayment: "api-ms-charge-payment",
    Microservice.billing:       "api-ms-billing",
    Microservice.email:         "api-ms-email",
    Microservice.nfe:           "api-ms-nfe",
    Microservice.taxPayment:    "api-ms-tax-payment",
    Microservice.holmes:        "api-ms-holmes",
    Microservice.approval:      "api-ms-approval",
}
